##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""
    Portal folders organize content.
"""
ADD_FOLDERS_PERMISSION = 'Add portal folders'
ADD_CONTENT_PERMISSION = 'Add portal content'

import Globals
import re
import base64
import marshal
import string

from OFS.Folder import Folder
from OFS.ObjectManager import REPLACEABLE
from Globals import HTMLFile
from AccessControl import getSecurityManager

#
#   HACK! HACK! HACK! HACK! HACK! HACK! HACK! HACK! HACK!
#
#   This registry needs to go away, and be replaced by a web-configurable,
#   instance-specific 'portal_types' tool.
#
_mime_type_registry = {}

def addPortalTypeHandler( MIMEType, klass ):
    """
        Set up a mapping from 'MIMEType' to 'klass', so that
        PortalFolder.PUT_factory knows what to make.  'klass' should be
        either an actual class object, whose __init__() takes no paramters
        (except 'self'), or a function taking no paramters.
    """
    _mime_type_registry[ MIMEType ] = klass

class PortalFolder( Folder ):
    """
        Implements portal content management, but not UI details.
    """
    meta_type = 'Portal Folder'

    __ac_permissions__=( ( 'Access contents information'
                         , ( 'getIcon'
                           ,
                           )
                         )
                       , ( ADD_FOLDERS_PERMISSION
                         , ( 'manage_addPortalFolder'
                           ,
                           )
                         , ( 'Owner','Manager')
                         )
                       , ( ADD_CONTENT_PERMISSION
                         , ( 'invokeFactory'
                           ,
                           )
                         )
                       )

    #
    #   Each module defining a PortalContent derivative registers here.
    #
    FOLDER_ACTION = 'manage_addProduct/PTKBase/manage_addPortalFolderForm'
    content_meta_types = ( { 'name'         : 'Portal Folder'
                           , 'action'       : FOLDER_ACTION
                           , 'permission'   : 'Add Portal Folders'
                           }
                         ,
                         )

    def __init__( self, id, title='' ):
        self.id = id
        self.title = title

    def all_meta_types( self ):

        names = {}
        types = []

        for cmt in self.content_meta_types:
            names[ cmt[ 'name' ] ] = 1
            types.append( cmt )

        superTypes = PortalFolder.inheritedAttribute(
            'all_meta_types' )( self )

        for smt in superTypes:
            if not names.has_key( smt[ 'name' ] ):
                names[ smt[ 'name' ] ] = 1
                types.append( smt )

        return tuple( types )
    
    def manage_addPortalFolder(self, id, title='',
                               REQUEST=None):
        """Add a new PortalFolder object with id *id*.
        """
        ob=PortalFolder( id, title )
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.folder_contents(
                self, REQUEST, portal_status_message="Folder added")
    

    listActions__roles__ = ()
    def listActions( self, info ):

        return ( { 'name'           : 'View'
                 , 'url'            : info.folder_url
                 , 'permissions'    : []
                 , 'category'       : 'folder'
                 }
               ,
               )

    getIcon__roles__ = None
    def getIcon(self):
        """
          Return the correct icon file
        """
        return getattr(self, 'icon', '/misc_/OFSP/Folder_icon.gif')


    listContentTypes__roles__ = None
    def listContentTypes( self ):
        """
           List all registered portal content types.
        """
        types =  map( lambda mtRec: mtRec['name'], self.content_meta_types )
        ttw_types = filter(None, getattr( self.portal_properties,
                                          'content_zclasses', [] ))
        types.extend( ttw_types )
        return types

    _morphSpec__roles__ = ()
    def _morphSpec( self, spec ):
        new_spec = []
        types = self.listContentTypes()
        if spec is not None:
            if type(spec) == type(''):
                spec = [ spec ]
            for meta_type in spec:
                if not meta_type in types:
                    raise 'PortalFolderError', ( '%s is not a content type'
                                               % meta_type )
                new_spec.append( meta_type )
        return new_spec or types
    
    def _filteredItems( self, ids, kw, REQUEST=None ):
        """
            Use 'kw' as filter on child objects indicated by 'ids',
            returning a sequence of ( id, obj ) tuples.
        """
        if REQUEST is not None:
            for key,value in REQUEST.items():
                if not kw.has_key(key):
                    kw[key] = value
        query = apply( ContentFilter, (), kw )
        result = []
        append = result.append
        get = self._getOb
        for id in ids:
            obj = get( id )
            if ((hasattr(obj, 'Type') and obj.Type() == 'Portal Folder'
                 and not REQUEST.get('FilterIncludesFolders', 0))
                or query( obj )):
                append( ( id, obj ) )
        return result

    contentIds__roles__ = None
    def contentIds( self, spec=None, REQUEST=None, **kw ):
        """
            Provide a filtered view onto 'objectIds', allowing only
            PortalFolders and PortalContent-derivatives to show through.

            If 'kw' passed, use them to filter the results further,
            qua the standard Zope filter interface.
        """
        spec = self._morphSpec( spec )
        ids = self.objectIds( spec )

        if not kw and not REQUEST:
            return ids
        
        return map( lambda item: item[0],
                    self._filteredItems( ids, kw, REQUEST ) )

    contentValues__roles__ = None
    def contentValues( self, spec=None, REQUEST=None, **kw ):
        """
            Provide a filtered view onto 'objectValues', allowing only
            PortalFolders and PortalContent-derivatives to show through.
        """
        spec = self._morphSpec( spec )
        if not kw and not REQUEST:
            return self.objectValues( spec )

        ids = self.objectIds( spec )
        return map( lambda item: item[1],
                    self._filteredItems( ids, kw, REQUEST ) )

    contentItems__roles__ = None
    def contentItems( self, spec=None, REQUEST=None, **kw ):
        """
            Provide a filtered view onto 'objectItems', allowing only
            PortalFolders and PortalContent-derivatives to show through.
        """
        spec = self._morphSpec( spec )
        if not kw and not REQUEST:
            return self.objectItems( spec )

        ids = self.objectIds( spec )
        return self._filteredItems( ids, kw, REQUEST )

    def Type( self ):
        """
             Implement dublin core type
        """
        return self.meta_type

    def createFilterString( self, decoded ):
        """
            Build Filter string for cookie munging from the REQUEST variables.
        """
        encoded = string.strip(base64.encodestring( marshal.dumps( decoded )))
        encoded = string.join(string.split(encoded, '\n'), '')
        return encoded

    def parseFilterString( self, REQUEST=None ):
        """
            Parse cookie string for using variables in dtml.
        """
        if REQUEST is None:
			REQUEST = {}
        # Set up defaults.
        decoded = { 'open': 0, 'Subject': ''
                  , 'Type': filter( lambda x: x <> 'Portal Folder'
                                  , self.listContentTypes()
                                  )
                  }
        # Parse the cookie
        if REQUEST.has_key('folderfilter'):
            decoded.update(marshal.loads(
                base64.decodestring(REQUEST['folderfilter'])))
        # Update the REQUEST (only if it doesn't have the key!).
        for key in ( 'open', 'Subject', 'Type' ):
            if REQUEST.has_key( key ) and (
                type(REQUEST[key]) in [type(""), type([]), type(1)]):
                decoded[ key ] = REQUEST[ key ]
            else:
                REQUEST[ key ] = decoded[ key ]
        # Make the cookie.
        REQUEST['encoded'] = self.createFilterString(decoded)

    def PUT_factory( self, name, typ, body ):
        """
            Dispatcher for PUT requests to non-existent IDs.  Returns
            an object of the appropriate type (or None, if we don't
            know what to do).
        """
        klass = _mime_type_registry.get( typ, None )
        if klass is None:
            return None
        obj = klass()
        obj.id = name
        return obj

    def invokeFactory(self, type_name, id, RESPONSE=None):
        '''
        Invokes the portal_types tool.
        '''
        pt = self.portal_types
        pt.constructContent(type_name, self, id, RESPONSE)        

    def _verifyObjectPaste(self, object, validate_src=1):
        # This assists the version in OFS.CopySupport.
        # It enables the clipboard to function correctly
        # with objects created by a multi-factory.
        if (hasattr(object, '__factory_meta_type__') and
            hasattr(self, 'all_meta_types')):
            mt = object.__factory_meta_type__
            method_name=None
            meta_types = self.all_meta_types
            if callable(meta_types): meta_types = meta_types()
            for d in meta_types:
                if d['name']==mt:
                    method_name=d['action']
                    break
            if method_name is not None:
                meth=self.unrestrictedTraverse(method_name)
                try:    parent=object.aq_inner.aq_parent
                except: parent=None
                if getSecurityManager().validate(None, parent, None, meth):
                    # Ensure the user is allowed to access the object on the
                    # clipboard.
                    if not validate_src:
                        return
                    try:    parent=object.aq_inner.aq_parent
                    except: parent=None
                    if getSecurityManager().validate(None, parent,
                                                     None, object):
                        return
                    id = object.id
                    if callable(id): id = id()
                    raise 'Unauthorized', id
                else:
                    raise 'Unauthorized', method_name
        PortalFolder.inheritedAttribute(
            '_verifyObjectPaste')(self, object, validate_src)


class ContentFilter:
    """
        Represent a predicate against a content object's metadata.
    """
    MARKER = []
    filterSubject = filterType = []
    def __init__( self
                , Title=MARKER
                , Creator=MARKER
                , Subject=MARKER
                , Description=MARKER
                , created=MARKER
                , created_usage='range:min'
                , modified=MARKER
                , modified_usage='range:min'
                , Type=MARKER
                , **Ignored
                ):

        self.predicates = []

        if Title is not self.MARKER: 
            self.filterTitle = Title
            self.predicates.append( lambda x, pat=re( Title ):
                                      pat.search( x.Title() ) )

        if Creator is not self.MARKER: 
            self.predicates.append( lambda x, pat=re( Creator ):
                                      pat.search( x.Creator() ) )

        if Subject and Subject is not self.MARKER: 
            self.filterSubject = Subject
            self.predicates.append( self.hasSubject )

        if Description is not self.MARKER: 
            self.predicates.append( lambda x, pat=re( Description ):
                                      pat.search( x.Description() ) )

        if created is not self.MARKER: 
            if created_usage == 'range:min':
                self.predicates.append( lambda x, cd=created:
                                          cd <= x.created() )
            if created_usage == 'range:max':
                self.predicates.append( lambda x, cd=created:
                                          cd >= x.created() )

        if modified is not self.MARKER: 
            if modified_usage == 'range:min':
                self.predicates.append( lambda x, md=modified:
                                          md <= x.modified() )
            if modified_usage == 'range:max':
                self.predicates.append( lambda x, md=modified:
                                          md >= x.modified() )

        if Type is not self.MARKER: 
            if type( Type ) == type( '' ):
                Type = [ Type ]
            self.filterType = Type
            self.predicates.append( lambda x, Type=Type:
                                      x.Type() in Type )

    def hasSubject( self, obj ):
        """
        Converts Subject string into a List for content filter view.
        """
        for sub in obj.Subject():
            if sub in self.Subject:
                return 1
        return 0

    def __call__( self, content ):

        for predicate in self.predicates:

            try:
                if not predicate( content ):
                    return 0
            except:
                return 0
        
        return 1

    def __str__( self ):
        """
        """
        return "Subject: %s; Type: %s" % ( self.filterSubject, self.filterType )
        

manage_addPortalFolder = PortalFolder.manage_addPortalFolder
manage_addPortalFolderForm = HTMLFile( 'folderAdd', globals() )

Globals.default__class_init__(PortalFolder)
